@@ -1,5 +1,5 @@
 /* dirmngr-client.c  -  A client for the dirmngr daemon
- *	Copyright (C) 2004 g10 Code GmbH
+ *	Copyright (C) 2004, 2007 g10 Code GmbH
  *	Copyright (C) 2002, 2003 Free Software Foundation, Inc.
  *
  * This file is part of DirMngr.
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -49,6 +48,8 @@ enum
   {
     oQuiet	  = 'q',
     oVerbose	  = 'v',
+    oLocal        = 'l',
+    oUrl          = 'u',
 
     oOCSP         = 500,
     oPing,
@@ -59,6 +60,7 @@ enum
     oSquidMode,
     oPEM,
     oEscapedPEM,
+    oForceDefaultResponder
   };
 
 
@@ -71,13 +73,16 @@ static ARGPARSE_OPTS opts[] = {
   { oCacheCert,"cache-cert",0, N_("add a certificate to the cache")},
   { oValidate, "validate",  0, N_("validate a certificate")},
   { oLookup,   "lookup",    0, N_("lookup a certificate")},
+  { oLocal,    "local",     0, N_("lookup only locally stored certificates")},
+  { oUrl,      "url",       0, N_("expect an URL for --lookup")},
   { oLoadCRL,  "load-crl",  0, N_("load a CRL into the dirmngr")},
   { oSquidMode,"squid-mode",0, N_("special mode for use by Squid")},
   { oPEM,      "pem",       0, N_("certificates are expected in PEM format")},
-  { 0 }
+  { oForceDefaultResponder, "force-default-responder", 0,
+    N_("force the use of the default OCSP responder")},
+  { 0, NULL, 0, NULL }
 };
  
-#define DEFAULT_SOCKET_NAME "/var/run/dirmngr/socket"
 
 /* The usual structure for the program flags.  */
 static struct
@@ -86,11 +91,13 @@ static struct
   int verbose;
   const char *dirmngr_program;
   int force_pipe_server;
+  int force_default_responder;
   int pem;
   int escaped_pem; /* PEM is additional percent encoded.  */
+  int url;         /* Expect an URL.  */
+  int local;       /* Lookup up only local certificates.  */
 
   int use_ocsp;
-
 } opt;
 
 
@@ -137,11 +144,10 @@ my_strusage (int level)
     case 11: p = "dirmngr-client (dirmngr)";
       break;
     case 13: p = VERSION; break;
-    case 14: p = "Copyright (C) 2004 g10 Code GmbH"; break;
+    case 14: p = "Copyright (C) " COPYRIGHT_YEAR_NAME; break;
     case 17: p = PRINTABLE_OS_NAME; break;
-    case 19: p =
-               _("Please report bugs to <gpa-dev@gnupg.org>.\n");
-      break;
+    case 19: p = _("Please report bugs to <@EMAIL@>.\n"); break;
+    case 49: p = PACKAGE_BUGREPORT; break;
     case 1:
     case 40: p =
                  _("Usage: dirmngr-client [options] "
@@ -193,10 +199,21 @@ main (int argc, char **argv )
   set_strusage (my_strusage);
   log_set_prefix ("dirmngr-client",
                   JNLIB_LOG_WITH_PREFIX); 
+
+  /* For W32 we need to initialize the socket subsystem.  Becuase we
+     don't use Pth we need to do this explicit. */
+#ifdef HAVE_W32_SYSTEM  
+ {
+   WSADATA wsadat;
+
+   WSAStartup (0x202, &wsadat);
+ }
+#endif /*HAVE_W32_SYSTEM*/
   
   /* Init Assuan.  */
   assuan_set_assuan_log_stream (log_get_stream ());
   assuan_set_assuan_log_prefix (log_get_prefix (NULL));
+  assuan_set_assuan_err_source (GPG_ERR_SOURCE_DEFAULT);
 
   /* Setup I18N. */
   i18n_init();
@@ -217,6 +234,8 @@ main (int argc, char **argv )
         case oCacheCert: cmd_cache_cert = 1; break;
         case oValidate: cmd_validate = 1; break;
         case oLookup: cmd_lookup = 1; break;
+        case oUrl: opt.url = 1; break;
+        case oLocal: opt.local = 1; break;
         case oLoadCRL: cmd_loadcrl = 1; break;
         case oPEM: opt.pem = 1; break;
         case oSquidMode: 
@@ -224,6 +243,7 @@ main (int argc, char **argv )
           opt.escaped_pem = 1;
           cmd_squid_mode = 1;
           break;
+        case oForceDefaultResponder: opt.force_default_responder = 1; break;
 
         default : pargs.err = 2; break;
 	}
@@ -355,7 +375,7 @@ main (int argc, char **argv )
         log_info (_("a dirmngr daemon is up and running\n"));
       return 0;
     }
-  else if (cmd_lookup||cmd_loadcrl||cmd_squid_mode)
+  else if (cmd_lookup|| cmd_loadcrl || cmd_squid_mode)
     return err? 1:0;
   else if (cmd_cache_cert)
     {
@@ -399,16 +419,18 @@ main (int argc, char **argv )
 
 
 /* Print status line from the assuan protocol.  */
-static assuan_error_t
+static gpg_error_t
 status_cb (void *opaque, const char *line)
 {
+  (void)opaque;
+
   if (opt.verbose > 2)
     log_info (_("got status: `%s'\n"), line);
   return 0;
 }
 
 /* Print data as retrieved by the lookup function.  */
-static assuan_error_t
+static gpg_error_t
 data_cb (void *opaque, const void *buffer, size_t length)
 {
   gpg_error_t err;
@@ -438,7 +460,7 @@ start_dirmngr (int only_daemon)
   infostr = opt.force_pipe_server? NULL : getenv ("DIRMNGR_INFO");
   if (only_daemon && (!infostr || !*infostr))
     {
-      infostr = DEFAULT_SOCKET_NAME;
+      infostr = xstrdup (default_socket_name ());
       try_default = 1;
     }
   if (!infostr || !*infostr)
@@ -475,7 +497,7 @@ start_dirmngr (int only_daemon)
       no_close_list[i] = -1;
 
       /* Connect to the agent and perform initial handshaking.  */
-      rc = assuan_pipe_connect (&ctx, opt.dirmngr_program, (char**)argv,
+      rc = assuan_pipe_connect (&ctx, opt.dirmngr_program, argv,
                                 no_close_list);
     }
   else /* Connect to a daemon.  */
@@ -517,7 +539,7 @@ start_dirmngr (int only_daemon)
 
       rc = assuan_socket_connect (&ctx, infostr, pid);
       xfree (infostr);
-      if (rc == ASSUAN_Connect_Failed && !only_daemon)
+      if (gpg_err_code(rc) == GPG_ERR_ASS_CONNECT_FAILED && !only_daemon)
         {
           log_error (_("can't connect to the dirmngr - trying fall back\n"));
           opt.force_pipe_server = 1;
@@ -528,7 +550,7 @@ start_dirmngr (int only_daemon)
   if (rc)
     {
       log_error (_("can't connect to the dirmngr: %s\n"),
-                 assuan_strerror (rc));
+                 gpg_strerror (rc));
       return NULL;
     }
 
@@ -744,47 +766,47 @@ read_certificate (const char *fname, unsigned char **rbuf, size_t *rbuflen)
 
 
 /* Callback for the inquire fiunction to send back the certificate.  */
-static assuan_error_t
+static gpg_error_t
 inq_cert (void *opaque, const char *line)
 {
   struct inq_cert_parm_s *parm = opaque;
-  assuan_error_t ae;
+  gpg_error_t err;
 
   if (!strncmp (line, "TARGETCERT", 10) && (line[10] == ' ' || !line[10]))
     {
-      ae = assuan_send_data (parm->ctx, parm->cert, parm->certlen);
+      err = assuan_send_data (parm->ctx, parm->cert, parm->certlen);
     }
   else if (!strncmp (line, "SENDCERT", 8) && (line[8] == ' ' || !line[8]))
     {
       /* We don't support this but dirmngr might ask for it.  So
          simply ignore it by sending back and empty value. */
-      ae = assuan_send_data (parm->ctx, NULL, 0);
+      err = assuan_send_data (parm->ctx, NULL, 0);
     }
   else if (!strncmp (line, "SENDCERT_SKI", 12)
            && (line[12]==' ' || !line[12]))
     {
       /* We don't support this but dirmngr might ask for it.  So
          simply ignore it by sending back an empty value. */
-      ae = assuan_send_data (parm->ctx, NULL, 0);
+      err = assuan_send_data (parm->ctx, NULL, 0);
     }
   else if (!strncmp (line, "SENDISSUERCERT", 14)
            && (line[14] == ' ' || !line[14]))
     {
       /* We don't support this but dirmngr might ask for it.  So
          simply ignore it by sending back an empty value. */
-      ae = assuan_send_data (parm->ctx, NULL, 0);
+      err = assuan_send_data (parm->ctx, NULL, 0);
     }
   else
     {
       log_info (_("unsupported inquiry `%s'\n"), line);
-      ae = ASSUAN_Inquire_Unknown;
+      err = gpg_error (GPG_ERR_ASS_UNKNOWN_INQUIRE);
       /* Note that this error will let assuan_transact terminate
          immediately instead of return the error to the caller.  It is
          not clear whether this is the desired behaviour - it may
          change in future. */
     }
 
-  return ae;
+  return err;
 }
 
 
@@ -793,7 +815,7 @@ inq_cert (void *opaque, const char *line)
 static gpg_error_t
 do_check (assuan_context_t ctx, const unsigned char *cert, size_t certlen)
 {
-  assuan_error_t ae;
+  gpg_error_t err;
   struct inq_cert_parm_s parm;
 
   memset (&parm, 0, sizeof parm);
@@ -801,12 +823,14 @@ do_check (assuan_context_t ctx, const unsigned char *cert, size_t certlen)
   parm.cert = cert;
   parm.certlen = certlen;
 
-  ae = assuan_transact (ctx, opt.use_ocsp? "CHECKOCSP":"CHECKCRL", NULL, NULL,
-                        inq_cert, &parm,
-                        status_cb, NULL);
+  err = assuan_transact (ctx, 
+                         (opt.use_ocsp && opt.force_default_responder
+                          ? "CHECKOCSP --force-default-responder" 
+                          : opt.use_ocsp? "CHECKOCSP" : "CHECKCRL"), 
+                         NULL, NULL, inq_cert, &parm, status_cb, NULL);
   if (opt.verbose > 1)
-    log_info ("response of dirmngr: %s\n", ae? assuan_strerror (ae): "okay");
-  return map_assuan_err (ae);
+    log_info ("response of dirmngr: %s\n", err? gpg_strerror (err): "okay");
+  return err;
 }
 
 /* Check the certificate CERT,CERTLEN for validity using a CRL or OCSP.
@@ -814,7 +838,7 @@ do_check (assuan_context_t ctx, const unsigned char *cert, size_t certlen)
 static gpg_error_t
 do_cache (assuan_context_t ctx, const unsigned char *cert, size_t certlen)
 {
-  assuan_error_t ae;
+  gpg_error_t err;
   struct inq_cert_parm_s parm;
 
   memset (&parm, 0, sizeof parm);
@@ -822,12 +846,12 @@ do_cache (assuan_context_t ctx, const unsigned char *cert, size_t certlen)
   parm.cert = cert;
   parm.certlen = certlen;
 
-  ae = assuan_transact (ctx, "CACHECERT", NULL, NULL,
+  err = assuan_transact (ctx, "CACHECERT", NULL, NULL,
                         inq_cert, &parm,
                         status_cb, NULL);
   if (opt.verbose > 1)
-    log_info ("response of dirmngr: %s\n", ae? assuan_strerror (ae): "okay");
-  return map_assuan_err (ae);
+    log_info ("response of dirmngr: %s\n", err? gpg_strerror (err): "okay");
+  return err;
 }
 
 /* Check the certificate CERT,CERTLEN for validity using dirmngrs
@@ -835,7 +859,7 @@ do_cache (assuan_context_t ctx, const unsigned char *cert, size_t certlen)
 static gpg_error_t
 do_validate (assuan_context_t ctx, const unsigned char *cert, size_t certlen)
 {
-  assuan_error_t ae;
+  gpg_error_t err;
   struct inq_cert_parm_s parm;
 
   memset (&parm, 0, sizeof parm);
@@ -843,19 +867,19 @@ do_validate (assuan_context_t ctx, const unsigned char *cert, size_t certlen)
   parm.cert = cert;
   parm.certlen = certlen;
 
-  ae = assuan_transact (ctx, "VALIDATE", NULL, NULL,
+  err = assuan_transact (ctx, "VALIDATE", NULL, NULL,
                         inq_cert, &parm,
                         status_cb, NULL);
   if (opt.verbose > 1)
-    log_info ("response of dirmngr: %s\n", ae? assuan_strerror (ae): "okay");
-  return map_assuan_err (ae);
+    log_info ("response of dirmngr: %s\n", err? gpg_strerror (err): "okay");
+  return err;
 }
 
 /* Load a CRL into the dirmngr.  */
 static gpg_error_t
 do_loadcrl (assuan_context_t ctx, const char *filename)
 {
-  assuan_error_t ae;
+  gpg_error_t err;
   const char *s;
   char *fname, *line, *p;
 
@@ -892,14 +916,14 @@ do_loadcrl (assuan_context_t ctx, const char *filename)
     }
   *p = 0;
 
-  ae = assuan_transact (ctx, line, NULL, NULL,
+  err = assuan_transact (ctx, line, NULL, NULL,
                         NULL, NULL,
                         status_cb, NULL);
   if (opt.verbose > 1)
-    log_info ("response of dirmngr: %s\n", ae? assuan_strerror (ae): "okay");
+    log_info ("response of dirmngr: %s\n", err? gpg_strerror (err): "okay");
   xfree (line);
   xfree (fname);
-  return map_assuan_err (ae);
+  return err;
 }
 
 
@@ -909,7 +933,6 @@ static gpg_error_t
 do_lookup (assuan_context_t ctx, const char *pattern)
 {
   gpg_error_t err;
-  assuan_error_t ae;
   const unsigned char *s;
   char *line, *p;
   struct b64state state;
@@ -921,9 +944,13 @@ do_lookup (assuan_context_t ctx, const char *pattern)
   if (err)
     return err;
 
-  line = xmalloc (10 + strlen (pattern)*3 + 1);
+  line = xmalloc (10 + 6 + 13 + strlen (pattern)*3 + 1);
 
   p = stpcpy (line, "LOOKUP ");
+  if (opt.url)
+    p = stpcpy (p, "--url ");
+  if (opt.local)
+    p = stpcpy (p, "--cache-only ");
   for (s=pattern; *s; s++)
     {
       if (*s < ' ' || *s == '+')
@@ -939,17 +966,17 @@ do_lookup (assuan_context_t ctx, const char *pattern)
   *p = 0;
 
 
-  ae = assuan_transact (ctx, line,
-                        data_cb, &state,
-                        NULL, NULL,
-                        status_cb, NULL);
+  err = assuan_transact (ctx, line,
+                         data_cb, &state,
+                         NULL, NULL,
+                         status_cb, NULL);
   if (opt.verbose > 1)
-    log_info ("response of dirmngr: %s\n", ae? assuan_strerror (ae): "okay");
+    log_info ("response of dirmngr: %s\n", err? gpg_strerror (err): "okay");
 
   err = b64enc_finish (&state);
 
   xfree (line);
-  return ae? map_assuan_err (ae) : err;
+  return err;
 }
 
 /* The body of an endless loop: Read a line from stdin, retrieve the
